# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.conferences import (
    action_hold_params,
    action_join_params,
    action_mute_params,
    action_play_params,
    action_stop_params,
    action_leave_params,
    action_speak_params,
    action_unhold_params,
    action_unmute_params,
    action_update_params,
    action_send_dtmf_params,
    action_record_stop_params,
    action_record_pause_params,
    action_record_start_params,
    action_record_resume_params,
    action_end_conference_params,
    action_gather_dtmf_audio_params,
)
from ...types.calls.loopcount_param import LoopcountParam
from ...types.conferences.action_hold_response import ActionHoldResponse
from ...types.conferences.action_join_response import ActionJoinResponse
from ...types.conferences.action_mute_response import ActionMuteResponse
from ...types.conferences.action_play_response import ActionPlayResponse
from ...types.conferences.action_stop_response import ActionStopResponse
from ...types.conferences.action_leave_response import ActionLeaveResponse
from ...types.conferences.action_speak_response import ActionSpeakResponse
from ...types.conferences.action_unhold_response import ActionUnholdResponse
from ...types.conferences.action_unmute_response import ActionUnmuteResponse
from ...types.conferences.action_update_response import ActionUpdateResponse
from ...types.conferences.action_send_dtmf_response import ActionSendDtmfResponse
from ...types.conferences.action_record_stop_response import ActionRecordStopResponse
from ...types.conferences.action_record_pause_response import ActionRecordPauseResponse
from ...types.conferences.action_record_start_response import ActionRecordStartResponse
from ...types.conferences.action_record_resume_response import ActionRecordResumeResponse
from ...types.conferences.action_end_conference_response import ActionEndConferenceResponse
from ...types.conferences.action_gather_dtmf_audio_response import ActionGatherDtmfAudioResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    """Conference command operations"""

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def update(
        self,
        id: str,
        *,
        call_control_id: str,
        supervisor_role: Literal["barge", "monitor", "none", "whisper"],
        command_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        whisper_call_control_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUpdateResponse:
        """
        Update conference participant supervisor_role

        Args:
          call_control_id: Unique identifier and token for controlling the call

          supervisor_role: Sets the participant as a supervisor for the conference. A conference can have
              multiple supervisors. "barge" means the supervisor enters the conference as a
              normal participant. This is the same as "none". "monitor" means the supervisor
              is muted but can hear all participants. "whisper" means that only the specified
              "whisper_call_control_ids" can hear the supervisor. Defaults to "none".

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          whisper_call_control_ids: Array of unique call_control_ids the supervisor can whisper to. If none
              provided, the supervisor will join the conference as a monitoring participant
              only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/update",
            body=maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "supervisor_role": supervisor_role,
                    "command_id": command_id,
                    "region": region,
                    "whisper_call_control_ids": whisper_call_control_ids,
                },
                action_update_params.ActionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUpdateResponse,
        )

    def end_conference(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEndConferenceResponse:
        """
        End a conference and terminate all active participants.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same conference.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/end",
            body=maybe_transform({"command_id": command_id}, action_end_conference_params.ActionEndConferenceParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEndConferenceResponse,
        )

    def gather_dtmf_audio(
        self,
        id: str,
        *,
        call_control_id: str,
        audio_url: str | Omit = omit,
        client_state: str | Omit = omit,
        gather_id: str | Omit = omit,
        initial_timeout_millis: int | Omit = omit,
        inter_digit_timeout_millis: int | Omit = omit,
        invalid_audio_url: str | Omit = omit,
        invalid_media_name: str | Omit = omit,
        maximum_digits: int | Omit = omit,
        maximum_tries: int | Omit = omit,
        media_name: str | Omit = omit,
        minimum_digits: int | Omit = omit,
        stop_playback_on_dtmf: bool | Omit = omit,
        terminating_digit: str | Omit = omit,
        timeout_millis: int | Omit = omit,
        valid_digits: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionGatherDtmfAudioResponse:
        """
        Play an audio file to a specific conference participant and gather DTMF input.

        Args:
          call_control_id: Unique identifier and token for controlling the call leg that will receive the
              gather prompt.

          audio_url: The URL of the audio file to play as the gather prompt. Must be WAV or MP3
              format.

          client_state: Use this field to add state to every subsequent webhook. Must be a valid Base-64
              encoded string.

          gather_id: Identifier for this gather command. Will be included in the gather ended
              webhook. Maximum 100 characters.

          initial_timeout_millis: Duration in milliseconds to wait for the first digit before timing out.

          inter_digit_timeout_millis: Duration in milliseconds to wait between digits.

          invalid_audio_url: URL of audio file to play when invalid input is received.

          invalid_media_name: Name of media file to play when invalid input is received.

          maximum_digits: Maximum number of digits to gather.

          maximum_tries: Maximum number of times to play the prompt if no input is received.

          media_name: The name of the media file uploaded to the Media Storage API to play as the
              gather prompt.

          minimum_digits: Minimum number of digits to gather.

          stop_playback_on_dtmf: Whether to stop the audio playback when a DTMF digit is received.

          terminating_digit: Digit that terminates gathering.

          timeout_millis: Duration in milliseconds to wait for input before timing out.

          valid_digits: Digits that are valid for gathering. All other digits will be ignored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/gather_using_audio",
            body=maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "audio_url": audio_url,
                    "client_state": client_state,
                    "gather_id": gather_id,
                    "initial_timeout_millis": initial_timeout_millis,
                    "inter_digit_timeout_millis": inter_digit_timeout_millis,
                    "invalid_audio_url": invalid_audio_url,
                    "invalid_media_name": invalid_media_name,
                    "maximum_digits": maximum_digits,
                    "maximum_tries": maximum_tries,
                    "media_name": media_name,
                    "minimum_digits": minimum_digits,
                    "stop_playback_on_dtmf": stop_playback_on_dtmf,
                    "terminating_digit": terminating_digit,
                    "timeout_millis": timeout_millis,
                    "valid_digits": valid_digits,
                },
                action_gather_dtmf_audio_params.ActionGatherDtmfAudioParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionGatherDtmfAudioResponse,
        )

    def hold(
        self,
        id: str,
        *,
        audio_url: str | Omit = omit,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        media_name: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionHoldResponse:
        """
        Hold a list of participants in a conference call

        Args:
          audio_url: The URL of a file to be played to the participants when they are put on hold.
              media_name and audio_url cannot be used together in one request.

          call_control_ids: List of unique identifiers and tokens for controlling the call. When empty all
              participants will be placed on hold.

          media_name: The media_name of a file to be played to the participants when they are put on
              hold. The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/hold",
            body=maybe_transform(
                {
                    "audio_url": audio_url,
                    "call_control_ids": call_control_ids,
                    "media_name": media_name,
                    "region": region,
                },
                action_hold_params.ActionHoldParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionHoldResponse,
        )

    def join(
        self,
        id: str,
        *,
        call_control_id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        hold: bool | Omit = omit,
        hold_audio_url: str | Omit = omit,
        hold_media_name: str | Omit = omit,
        mute: bool | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        soft_end_conference_on_exit: bool | Omit = omit,
        start_conference_on_enter: bool | Omit = omit,
        supervisor_role: Literal["barge", "monitor", "none", "whisper"] | Omit = omit,
        whisper_call_control_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionJoinResponse:
        """Join an existing call leg to a conference.

        Issue the Join Conference command
        with the conference ID in the path and the `call_control_id` of the leg you wish
        to join to the conference as an attribute. The conference can have up to a
        certain amount of active participants, as set by the `max_participants`
        parameter in conference creation request.

        **Expected Webhooks:**

        - `conference.participant.joined`
        - `conference.participant.left`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          beep_enabled: Whether a beep sound should be played when the participant joins and/or leaves
              the conference. Can be used to override the conference-level setting.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string. Please note that the client_state will be updated for
              the participient call leg and the change will not affect conferencing webhooks
              unless the participient is the owner of the conference.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          end_conference_on_exit: Whether the conference should end and all remaining participants be hung up
              after the participant leaves the conference. Defaults to "false".

          hold: Whether the participant should be put on hold immediately after joining the
              conference. Defaults to "false".

          hold_audio_url: The URL of a file to be played to the participant when they are put on hold
              after joining the conference. hold_media_name and hold_audio_url cannot be used
              together in one request. Takes effect only when "start_conference_on_create" is
              set to "false". This property takes effect only if "hold" is set to "true".

          hold_media_name: The media_name of a file to be played to the participant when they are put on
              hold after joining the conference. The media_name must point to a file
              previously uploaded to api.telnyx.com/v2/media by the same user/organization.
              The file must either be a WAV or MP3 file. Takes effect only when
              "start_conference_on_create" is set to "false". This property takes effect only
              if "hold" is set to "true".

          mute: Whether the participant should be muted immediately after joining the
              conference. Defaults to "false".

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          soft_end_conference_on_exit: Whether the conference should end after the participant leaves the conference.
              NOTE this doesn't hang up the other participants. Defaults to "false".

          start_conference_on_enter: Whether the conference should be started after the participant joins the
              conference. Defaults to "false".

          supervisor_role: Sets the joining participant as a supervisor for the conference. A conference
              can have multiple supervisors. "barge" means the supervisor enters the
              conference as a normal participant. This is the same as "none". "monitor" means
              the supervisor is muted but can hear all participants. "whisper" means that only
              the specified "whisper_call_control_ids" can hear the supervisor. Defaults to
              "none".

          whisper_call_control_ids: Array of unique call_control_ids the joining supervisor can whisper to. If none
              provided, the supervisor will join the conference as a monitoring participant
              only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/join",
            body=maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "beep_enabled": beep_enabled,
                    "client_state": client_state,
                    "command_id": command_id,
                    "end_conference_on_exit": end_conference_on_exit,
                    "hold": hold,
                    "hold_audio_url": hold_audio_url,
                    "hold_media_name": hold_media_name,
                    "mute": mute,
                    "region": region,
                    "soft_end_conference_on_exit": soft_end_conference_on_exit,
                    "start_conference_on_enter": start_conference_on_enter,
                    "supervisor_role": supervisor_role,
                    "whisper_call_control_ids": whisper_call_control_ids,
                },
                action_join_params.ActionJoinParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionJoinResponse,
        )

    def leave(
        self,
        id: str,
        *,
        call_control_id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        command_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionLeaveResponse:
        """
        Removes a call leg from a conference and moves it back to parked state.

        **Expected Webhooks:**

        - `conference.participant.left`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          beep_enabled: Whether a beep sound should be played when the participant leaves the
              conference. Can be used to override the conference-level setting.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/leave",
            body=maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "beep_enabled": beep_enabled,
                    "command_id": command_id,
                    "region": region,
                },
                action_leave_params.ActionLeaveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionLeaveResponse,
        )

    def mute(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionMuteResponse:
        """
        Mute a list of participants in a conference call

        Args:
          call_control_ids: Array of unique identifiers and tokens for controlling the call. When empty all
              participants will be muted.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/mute",
            body=maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_mute_params.ActionMuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionMuteResponse,
        )

    def play(
        self,
        id: str,
        *,
        audio_url: str | Omit = omit,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        loop: LoopcountParam | Omit = omit,
        media_name: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionPlayResponse:
        """
        Play audio to all or some participants on a conference call.

        Args:
          audio_url: The URL of a file to be played back in the conference. media_name and audio_url
              cannot be used together in one request.

          call_control_ids: List of call control ids identifying participants the audio file should be
              played to. If not given, the audio file will be played to the entire conference.

          loop: The number of times the audio file should be played. If supplied, the value must
              be an integer between 1 and 100, or the special string `infinity` for an endless
              loop.

          media_name: The media_name of a file to be played back in the conference. The media_name
              must point to a file previously uploaded to api.telnyx.com/v2/media by the same
              user/organization. The file must either be a WAV or MP3 file.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/play",
            body=maybe_transform(
                {
                    "audio_url": audio_url,
                    "call_control_ids": call_control_ids,
                    "loop": loop,
                    "media_name": media_name,
                    "region": region,
                },
                action_play_params.ActionPlayParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionPlayResponse,
        )

    def record_pause(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordPauseResponse:
        """
        Pause conference recording.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Use this field to pause specific recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/record_pause",
            body=maybe_transform(
                {
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_pause_params.ActionRecordPauseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordPauseResponse,
        )

    def record_resume(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordResumeResponse:
        """
        Resume conference recording.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Use this field to resume specific recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/record_resume",
            body=maybe_transform(
                {
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_resume_params.ActionRecordResumeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordResumeResponse,
        )

    def record_start(
        self,
        id: str,
        *,
        format: Literal["wav", "mp3"],
        channels: Literal["single", "dual"] | Omit = omit,
        command_id: str | Omit = omit,
        custom_file_name: str | Omit = omit,
        play_beep: bool | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        trim: Literal["trim-silence"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordStartResponse:
        """Start recording the conference.

        Recording will stop on conference end, or via
        the Stop Recording command.

        **Expected Webhooks:**

        - `conference.recording.saved`

        Args:
          format: The audio file format used when storing the conference recording. Can be either
              `mp3` or `wav`.

          channels: When `dual`, final audio file will be stereo recorded with the conference
              creator on the first channel, and the rest on the second channel.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `conference_id`.

          custom_file_name: The custom recording file name to be used instead of the default `call_leg_id`.
              Telnyx will still add a Unix timestamp suffix.

          play_beep: If enabled, a beep sound will be played at the start of a recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          trim: When set to `trim-silence`, silence will be removed from the beginning and end
              of the recording.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/record_start",
            body=maybe_transform(
                {
                    "format": format,
                    "channels": channels,
                    "command_id": command_id,
                    "custom_file_name": custom_file_name,
                    "play_beep": play_beep,
                    "region": region,
                    "trim": trim,
                },
                action_record_start_params.ActionRecordStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordStartResponse,
        )

    def record_stop(
        self,
        id: str,
        *,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordStopResponse:
        """
        Stop recording the conference.

        **Expected Webhooks:**

        - `conference.recording.saved`

        Args:
          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Uniquely identifies the resource.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/record_stop",
            body=maybe_transform(
                {
                    "client_state": client_state,
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_stop_params.ActionRecordStopParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordStopResponse,
        )

    def send_dtmf(
        self,
        id: str,
        *,
        digits: str,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        client_state: str | Omit = omit,
        duration_millis: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSendDtmfResponse:
        """
        Send DTMF tones to one or more conference participants.

        Args:
          digits: DTMF digits to send. Valid characters: 0-9, A-D, \\**, #, w (0.5s pause), W (1s
              pause).

          call_control_ids: Array of participant call control IDs to send DTMF to. When empty, DTMF will be
              sent to all participants.

          client_state: Use this field to add state to every subsequent webhook. Must be a valid Base-64
              encoded string.

          duration_millis: Duration of each DTMF digit in milliseconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/send_dtmf",
            body=maybe_transform(
                {
                    "digits": digits,
                    "call_control_ids": call_control_ids,
                    "client_state": client_state,
                    "duration_millis": duration_millis,
                },
                action_send_dtmf_params.ActionSendDtmfParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSendDtmfResponse,
        )

    def speak(
        self,
        id: str,
        *,
        payload: str,
        voice: str,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        command_id: str | Omit = omit,
        language: Literal[
            "arb",
            "cmn-CN",
            "cy-GB",
            "da-DK",
            "de-DE",
            "en-AU",
            "en-GB",
            "en-GB-WLS",
            "en-IN",
            "en-US",
            "es-ES",
            "es-MX",
            "es-US",
            "fr-CA",
            "fr-FR",
            "hi-IN",
            "is-IS",
            "it-IT",
            "ja-JP",
            "ko-KR",
            "nb-NO",
            "nl-NL",
            "pl-PL",
            "pt-BR",
            "pt-PT",
            "ro-RO",
            "ru-RU",
            "sv-SE",
            "tr-TR",
        ]
        | Omit = omit,
        payload_type: Literal["text", "ssml"] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        voice_settings: action_speak_params.VoiceSettings | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSpeakResponse:
        """
        Convert text to speech and play it to all or some participants.

        Args:
          payload: The text or SSML to be converted into speech. There is a 3,000 character limit.

          voice: Specifies the voice used in speech synthesis.

              - Define voices using the format `<Provider>.<Model>.<VoiceId>`. Specifying only
                the provider will give default values for voice_id and model_id.

                **Supported Providers:**

              - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
                voices, which provide more realistic, human-like speech, append `-Neural` to
                the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
                [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
                for compatibility.
              - **Azure:** Use `Azure.<VoiceId>` (e.g., `Azure.en-CA-ClaraNeural`,
                `Azure.en-US-BrianMultilingualNeural`,
                `Azure.en-US-Ava:DragonHDLatestNeural`). For a complete list of voices, go to
                [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery). Use
                `voice_settings` to configure custom deployments, regions, or API keys.
              - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
                `ElevenLabs.eleven_multilingual_v2.21m00Tcm4TlvDq8ikWAM`). The `ModelId` part
                is optional. To use ElevenLabs, you must provide your ElevenLabs API key as an
                integration identifier secret in
                `"voice_settings": {"api_key_ref": "<secret_identifier>"}`. See
                [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
                for details. Check
                [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
              - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>` (e.g., `Telnyx.KokoroTTS.af`).
                Use `voice_settings` to configure voice_speed and other synthesis parameters.
              - **Minimax:** Use `Minimax.<ModelId>.<VoiceId>` (e.g.,
                `Minimax.speech-02-hd.Wise_Woman`). Supported models: `speech-02-turbo`,
                `speech-02-hd`, `speech-2.6-turbo`, `speech-2.8-turbo`. Use `voice_settings`
                to configure speed, volume, pitch, and language_boost.
              - **Rime:** Use `Rime.<model_id>.<voice_id>` (e.g., `Rime.Arcana.cove`).
                Supported model_ids: `Arcana`, `Mist`. Use `voice_settings` to configure
                voice_speed.
              - **Resemble:** Use `Resemble.Turbo.<voice_id>` (e.g.,
                `Resemble.Turbo.my_voice`). Only `Turbo` model is supported. Use
                `voice_settings` to configure precision, sample_rate, and format.
              - **Inworld:** Use `Inworld.<ModelId>.<VoiceId>` (e.g., `Inworld.Mini.Loretta`,
                `Inworld.Max.Oliver`). Supported models: `Mini`, `Max`.

              For service_level basic, you may define the gender of the speaker (male or
              female).

          call_control_ids: Call Control IDs of participants who will hear the spoken text. When empty all
              participants will hear the spoken text.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          language: The language you want spoken. This parameter is ignored when a `Polly.*` voice
              is specified.

          payload_type: The type of the provided payload. The payload can either be plain text, or
              Speech Synthesis Markup Language (SSML).

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          voice_settings: The settings associated with the voice selected

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/speak",
            body=maybe_transform(
                {
                    "payload": payload,
                    "voice": voice,
                    "call_control_ids": call_control_ids,
                    "command_id": command_id,
                    "language": language,
                    "payload_type": payload_type,
                    "region": region,
                    "voice_settings": voice_settings,
                },
                action_speak_params.ActionSpeakParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSpeakResponse,
        )

    def stop(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionStopResponse:
        """
        Stop audio being played to all or some participants on a conference call.

        Args:
          call_control_ids: List of call control ids identifying participants the audio file should stop be
              played to. If not given, the audio will be stoped to the entire conference.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/stop",
            body=maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_stop_params.ActionStopParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionStopResponse,
        )

    def unhold(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str],
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnholdResponse:
        """
        Unhold a list of participants in a conference call

        Args:
          call_control_ids: List of unique identifiers and tokens for controlling the call. Enter each call
              control ID to be unheld.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/unhold",
            body=maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_unhold_params.ActionUnholdParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnholdResponse,
        )

    def unmute(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnmuteResponse:
        """
        Unmute a list of participants in a conference call

        Args:
          call_control_ids: List of unique identifiers and tokens for controlling the call. Enter each call
              control ID to be unmuted. When empty all participants will be unmuted.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/conferences/{id}/actions/unmute",
            body=maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_unmute_params.ActionUnmuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnmuteResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    """Conference command operations"""

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def update(
        self,
        id: str,
        *,
        call_control_id: str,
        supervisor_role: Literal["barge", "monitor", "none", "whisper"],
        command_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        whisper_call_control_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUpdateResponse:
        """
        Update conference participant supervisor_role

        Args:
          call_control_id: Unique identifier and token for controlling the call

          supervisor_role: Sets the participant as a supervisor for the conference. A conference can have
              multiple supervisors. "barge" means the supervisor enters the conference as a
              normal participant. This is the same as "none". "monitor" means the supervisor
              is muted but can hear all participants. "whisper" means that only the specified
              "whisper_call_control_ids" can hear the supervisor. Defaults to "none".

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          whisper_call_control_ids: Array of unique call_control_ids the supervisor can whisper to. If none
              provided, the supervisor will join the conference as a monitoring participant
              only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/update",
            body=await async_maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "supervisor_role": supervisor_role,
                    "command_id": command_id,
                    "region": region,
                    "whisper_call_control_ids": whisper_call_control_ids,
                },
                action_update_params.ActionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUpdateResponse,
        )

    async def end_conference(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEndConferenceResponse:
        """
        End a conference and terminate all active participants.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same conference.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/end",
            body=await async_maybe_transform(
                {"command_id": command_id}, action_end_conference_params.ActionEndConferenceParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEndConferenceResponse,
        )

    async def gather_dtmf_audio(
        self,
        id: str,
        *,
        call_control_id: str,
        audio_url: str | Omit = omit,
        client_state: str | Omit = omit,
        gather_id: str | Omit = omit,
        initial_timeout_millis: int | Omit = omit,
        inter_digit_timeout_millis: int | Omit = omit,
        invalid_audio_url: str | Omit = omit,
        invalid_media_name: str | Omit = omit,
        maximum_digits: int | Omit = omit,
        maximum_tries: int | Omit = omit,
        media_name: str | Omit = omit,
        minimum_digits: int | Omit = omit,
        stop_playback_on_dtmf: bool | Omit = omit,
        terminating_digit: str | Omit = omit,
        timeout_millis: int | Omit = omit,
        valid_digits: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionGatherDtmfAudioResponse:
        """
        Play an audio file to a specific conference participant and gather DTMF input.

        Args:
          call_control_id: Unique identifier and token for controlling the call leg that will receive the
              gather prompt.

          audio_url: The URL of the audio file to play as the gather prompt. Must be WAV or MP3
              format.

          client_state: Use this field to add state to every subsequent webhook. Must be a valid Base-64
              encoded string.

          gather_id: Identifier for this gather command. Will be included in the gather ended
              webhook. Maximum 100 characters.

          initial_timeout_millis: Duration in milliseconds to wait for the first digit before timing out.

          inter_digit_timeout_millis: Duration in milliseconds to wait between digits.

          invalid_audio_url: URL of audio file to play when invalid input is received.

          invalid_media_name: Name of media file to play when invalid input is received.

          maximum_digits: Maximum number of digits to gather.

          maximum_tries: Maximum number of times to play the prompt if no input is received.

          media_name: The name of the media file uploaded to the Media Storage API to play as the
              gather prompt.

          minimum_digits: Minimum number of digits to gather.

          stop_playback_on_dtmf: Whether to stop the audio playback when a DTMF digit is received.

          terminating_digit: Digit that terminates gathering.

          timeout_millis: Duration in milliseconds to wait for input before timing out.

          valid_digits: Digits that are valid for gathering. All other digits will be ignored.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/gather_using_audio",
            body=await async_maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "audio_url": audio_url,
                    "client_state": client_state,
                    "gather_id": gather_id,
                    "initial_timeout_millis": initial_timeout_millis,
                    "inter_digit_timeout_millis": inter_digit_timeout_millis,
                    "invalid_audio_url": invalid_audio_url,
                    "invalid_media_name": invalid_media_name,
                    "maximum_digits": maximum_digits,
                    "maximum_tries": maximum_tries,
                    "media_name": media_name,
                    "minimum_digits": minimum_digits,
                    "stop_playback_on_dtmf": stop_playback_on_dtmf,
                    "terminating_digit": terminating_digit,
                    "timeout_millis": timeout_millis,
                    "valid_digits": valid_digits,
                },
                action_gather_dtmf_audio_params.ActionGatherDtmfAudioParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionGatherDtmfAudioResponse,
        )

    async def hold(
        self,
        id: str,
        *,
        audio_url: str | Omit = omit,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        media_name: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionHoldResponse:
        """
        Hold a list of participants in a conference call

        Args:
          audio_url: The URL of a file to be played to the participants when they are put on hold.
              media_name and audio_url cannot be used together in one request.

          call_control_ids: List of unique identifiers and tokens for controlling the call. When empty all
              participants will be placed on hold.

          media_name: The media_name of a file to be played to the participants when they are put on
              hold. The media_name must point to a file previously uploaded to
              api.telnyx.com/v2/media by the same user/organization. The file must either be a
              WAV or MP3 file.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/hold",
            body=await async_maybe_transform(
                {
                    "audio_url": audio_url,
                    "call_control_ids": call_control_ids,
                    "media_name": media_name,
                    "region": region,
                },
                action_hold_params.ActionHoldParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionHoldResponse,
        )

    async def join(
        self,
        id: str,
        *,
        call_control_id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        end_conference_on_exit: bool | Omit = omit,
        hold: bool | Omit = omit,
        hold_audio_url: str | Omit = omit,
        hold_media_name: str | Omit = omit,
        mute: bool | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        soft_end_conference_on_exit: bool | Omit = omit,
        start_conference_on_enter: bool | Omit = omit,
        supervisor_role: Literal["barge", "monitor", "none", "whisper"] | Omit = omit,
        whisper_call_control_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionJoinResponse:
        """Join an existing call leg to a conference.

        Issue the Join Conference command
        with the conference ID in the path and the `call_control_id` of the leg you wish
        to join to the conference as an attribute. The conference can have up to a
        certain amount of active participants, as set by the `max_participants`
        parameter in conference creation request.

        **Expected Webhooks:**

        - `conference.participant.joined`
        - `conference.participant.left`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          beep_enabled: Whether a beep sound should be played when the participant joins and/or leaves
              the conference. Can be used to override the conference-level setting.

          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string. Please note that the client_state will be updated for
              the participient call leg and the change will not affect conferencing webhooks
              unless the participient is the owner of the conference.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          end_conference_on_exit: Whether the conference should end and all remaining participants be hung up
              after the participant leaves the conference. Defaults to "false".

          hold: Whether the participant should be put on hold immediately after joining the
              conference. Defaults to "false".

          hold_audio_url: The URL of a file to be played to the participant when they are put on hold
              after joining the conference. hold_media_name and hold_audio_url cannot be used
              together in one request. Takes effect only when "start_conference_on_create" is
              set to "false". This property takes effect only if "hold" is set to "true".

          hold_media_name: The media_name of a file to be played to the participant when they are put on
              hold after joining the conference. The media_name must point to a file
              previously uploaded to api.telnyx.com/v2/media by the same user/organization.
              The file must either be a WAV or MP3 file. Takes effect only when
              "start_conference_on_create" is set to "false". This property takes effect only
              if "hold" is set to "true".

          mute: Whether the participant should be muted immediately after joining the
              conference. Defaults to "false".

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          soft_end_conference_on_exit: Whether the conference should end after the participant leaves the conference.
              NOTE this doesn't hang up the other participants. Defaults to "false".

          start_conference_on_enter: Whether the conference should be started after the participant joins the
              conference. Defaults to "false".

          supervisor_role: Sets the joining participant as a supervisor for the conference. A conference
              can have multiple supervisors. "barge" means the supervisor enters the
              conference as a normal participant. This is the same as "none". "monitor" means
              the supervisor is muted but can hear all participants. "whisper" means that only
              the specified "whisper_call_control_ids" can hear the supervisor. Defaults to
              "none".

          whisper_call_control_ids: Array of unique call_control_ids the joining supervisor can whisper to. If none
              provided, the supervisor will join the conference as a monitoring participant
              only.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/join",
            body=await async_maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "beep_enabled": beep_enabled,
                    "client_state": client_state,
                    "command_id": command_id,
                    "end_conference_on_exit": end_conference_on_exit,
                    "hold": hold,
                    "hold_audio_url": hold_audio_url,
                    "hold_media_name": hold_media_name,
                    "mute": mute,
                    "region": region,
                    "soft_end_conference_on_exit": soft_end_conference_on_exit,
                    "start_conference_on_enter": start_conference_on_enter,
                    "supervisor_role": supervisor_role,
                    "whisper_call_control_ids": whisper_call_control_ids,
                },
                action_join_params.ActionJoinParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionJoinResponse,
        )

    async def leave(
        self,
        id: str,
        *,
        call_control_id: str,
        beep_enabled: Literal["always", "never", "on_enter", "on_exit"] | Omit = omit,
        command_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionLeaveResponse:
        """
        Removes a call leg from a conference and moves it back to parked state.

        **Expected Webhooks:**

        - `conference.participant.left`

        Args:
          call_control_id: Unique identifier and token for controlling the call

          beep_enabled: Whether a beep sound should be played when the participant leaves the
              conference. Can be used to override the conference-level setting.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/leave",
            body=await async_maybe_transform(
                {
                    "call_control_id": call_control_id,
                    "beep_enabled": beep_enabled,
                    "command_id": command_id,
                    "region": region,
                },
                action_leave_params.ActionLeaveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionLeaveResponse,
        )

    async def mute(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionMuteResponse:
        """
        Mute a list of participants in a conference call

        Args:
          call_control_ids: Array of unique identifiers and tokens for controlling the call. When empty all
              participants will be muted.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/mute",
            body=await async_maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_mute_params.ActionMuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionMuteResponse,
        )

    async def play(
        self,
        id: str,
        *,
        audio_url: str | Omit = omit,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        loop: LoopcountParam | Omit = omit,
        media_name: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionPlayResponse:
        """
        Play audio to all or some participants on a conference call.

        Args:
          audio_url: The URL of a file to be played back in the conference. media_name and audio_url
              cannot be used together in one request.

          call_control_ids: List of call control ids identifying participants the audio file should be
              played to. If not given, the audio file will be played to the entire conference.

          loop: The number of times the audio file should be played. If supplied, the value must
              be an integer between 1 and 100, or the special string `infinity` for an endless
              loop.

          media_name: The media_name of a file to be played back in the conference. The media_name
              must point to a file previously uploaded to api.telnyx.com/v2/media by the same
              user/organization. The file must either be a WAV or MP3 file.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/play",
            body=await async_maybe_transform(
                {
                    "audio_url": audio_url,
                    "call_control_ids": call_control_ids,
                    "loop": loop,
                    "media_name": media_name,
                    "region": region,
                },
                action_play_params.ActionPlayParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionPlayResponse,
        )

    async def record_pause(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordPauseResponse:
        """
        Pause conference recording.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Use this field to pause specific recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/record_pause",
            body=await async_maybe_transform(
                {
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_pause_params.ActionRecordPauseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordPauseResponse,
        )

    async def record_resume(
        self,
        id: str,
        *,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordResumeResponse:
        """
        Resume conference recording.

        Args:
          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Use this field to resume specific recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/record_resume",
            body=await async_maybe_transform(
                {
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_resume_params.ActionRecordResumeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordResumeResponse,
        )

    async def record_start(
        self,
        id: str,
        *,
        format: Literal["wav", "mp3"],
        channels: Literal["single", "dual"] | Omit = omit,
        command_id: str | Omit = omit,
        custom_file_name: str | Omit = omit,
        play_beep: bool | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        trim: Literal["trim-silence"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordStartResponse:
        """Start recording the conference.

        Recording will stop on conference end, or via
        the Stop Recording command.

        **Expected Webhooks:**

        - `conference.recording.saved`

        Args:
          format: The audio file format used when storing the conference recording. Can be either
              `mp3` or `wav`.

          channels: When `dual`, final audio file will be stereo recorded with the conference
              creator on the first channel, and the rest on the second channel.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `conference_id`.

          custom_file_name: The custom recording file name to be used instead of the default `call_leg_id`.
              Telnyx will still add a Unix timestamp suffix.

          play_beep: If enabled, a beep sound will be played at the start of a recording.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          trim: When set to `trim-silence`, silence will be removed from the beginning and end
              of the recording.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/record_start",
            body=await async_maybe_transform(
                {
                    "format": format,
                    "channels": channels,
                    "command_id": command_id,
                    "custom_file_name": custom_file_name,
                    "play_beep": play_beep,
                    "region": region,
                    "trim": trim,
                },
                action_record_start_params.ActionRecordStartParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordStartResponse,
        )

    async def record_stop(
        self,
        id: str,
        *,
        client_state: str | Omit = omit,
        command_id: str | Omit = omit,
        recording_id: str | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRecordStopResponse:
        """
        Stop recording the conference.

        **Expected Webhooks:**

        - `conference.recording.saved`

        Args:
          client_state: Use this field to add state to every subsequent webhook. It must be a valid
              Base-64 encoded string.

          command_id: Use this field to avoid duplicate commands. Telnyx will ignore any command with
              the same `command_id` for the same `call_control_id`.

          recording_id: Uniquely identifies the resource.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/record_stop",
            body=await async_maybe_transform(
                {
                    "client_state": client_state,
                    "command_id": command_id,
                    "recording_id": recording_id,
                    "region": region,
                },
                action_record_stop_params.ActionRecordStopParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRecordStopResponse,
        )

    async def send_dtmf(
        self,
        id: str,
        *,
        digits: str,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        client_state: str | Omit = omit,
        duration_millis: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSendDtmfResponse:
        """
        Send DTMF tones to one or more conference participants.

        Args:
          digits: DTMF digits to send. Valid characters: 0-9, A-D, \\**, #, w (0.5s pause), W (1s
              pause).

          call_control_ids: Array of participant call control IDs to send DTMF to. When empty, DTMF will be
              sent to all participants.

          client_state: Use this field to add state to every subsequent webhook. Must be a valid Base-64
              encoded string.

          duration_millis: Duration of each DTMF digit in milliseconds.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/send_dtmf",
            body=await async_maybe_transform(
                {
                    "digits": digits,
                    "call_control_ids": call_control_ids,
                    "client_state": client_state,
                    "duration_millis": duration_millis,
                },
                action_send_dtmf_params.ActionSendDtmfParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSendDtmfResponse,
        )

    async def speak(
        self,
        id: str,
        *,
        payload: str,
        voice: str,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        command_id: str | Omit = omit,
        language: Literal[
            "arb",
            "cmn-CN",
            "cy-GB",
            "da-DK",
            "de-DE",
            "en-AU",
            "en-GB",
            "en-GB-WLS",
            "en-IN",
            "en-US",
            "es-ES",
            "es-MX",
            "es-US",
            "fr-CA",
            "fr-FR",
            "hi-IN",
            "is-IS",
            "it-IT",
            "ja-JP",
            "ko-KR",
            "nb-NO",
            "nl-NL",
            "pl-PL",
            "pt-BR",
            "pt-PT",
            "ro-RO",
            "ru-RU",
            "sv-SE",
            "tr-TR",
        ]
        | Omit = omit,
        payload_type: Literal["text", "ssml"] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        voice_settings: action_speak_params.VoiceSettings | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSpeakResponse:
        """
        Convert text to speech and play it to all or some participants.

        Args:
          payload: The text or SSML to be converted into speech. There is a 3,000 character limit.

          voice: Specifies the voice used in speech synthesis.

              - Define voices using the format `<Provider>.<Model>.<VoiceId>`. Specifying only
                the provider will give default values for voice_id and model_id.

                **Supported Providers:**

              - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
                voices, which provide more realistic, human-like speech, append `-Neural` to
                the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
                [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
                for compatibility.
              - **Azure:** Use `Azure.<VoiceId>` (e.g., `Azure.en-CA-ClaraNeural`,
                `Azure.en-US-BrianMultilingualNeural`,
                `Azure.en-US-Ava:DragonHDLatestNeural`). For a complete list of voices, go to
                [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery). Use
                `voice_settings` to configure custom deployments, regions, or API keys.
              - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
                `ElevenLabs.eleven_multilingual_v2.21m00Tcm4TlvDq8ikWAM`). The `ModelId` part
                is optional. To use ElevenLabs, you must provide your ElevenLabs API key as an
                integration identifier secret in
                `"voice_settings": {"api_key_ref": "<secret_identifier>"}`. See
                [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
                for details. Check
                [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
              - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>` (e.g., `Telnyx.KokoroTTS.af`).
                Use `voice_settings` to configure voice_speed and other synthesis parameters.
              - **Minimax:** Use `Minimax.<ModelId>.<VoiceId>` (e.g.,
                `Minimax.speech-02-hd.Wise_Woman`). Supported models: `speech-02-turbo`,
                `speech-02-hd`, `speech-2.6-turbo`, `speech-2.8-turbo`. Use `voice_settings`
                to configure speed, volume, pitch, and language_boost.
              - **Rime:** Use `Rime.<model_id>.<voice_id>` (e.g., `Rime.Arcana.cove`).
                Supported model_ids: `Arcana`, `Mist`. Use `voice_settings` to configure
                voice_speed.
              - **Resemble:** Use `Resemble.Turbo.<voice_id>` (e.g.,
                `Resemble.Turbo.my_voice`). Only `Turbo` model is supported. Use
                `voice_settings` to configure precision, sample_rate, and format.
              - **Inworld:** Use `Inworld.<ModelId>.<VoiceId>` (e.g., `Inworld.Mini.Loretta`,
                `Inworld.Max.Oliver`). Supported models: `Mini`, `Max`.

              For service_level basic, you may define the gender of the speaker (male or
              female).

          call_control_ids: Call Control IDs of participants who will hear the spoken text. When empty all
              participants will hear the spoken text.

          command_id: Use this field to avoid execution of duplicate commands. Telnyx will ignore
              subsequent commands with the same `command_id` as one that has already been
              executed.

          language: The language you want spoken. This parameter is ignored when a `Polly.*` voice
              is specified.

          payload_type: The type of the provided payload. The payload can either be plain text, or
              Speech Synthesis Markup Language (SSML).

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          voice_settings: The settings associated with the voice selected

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/speak",
            body=await async_maybe_transform(
                {
                    "payload": payload,
                    "voice": voice,
                    "call_control_ids": call_control_ids,
                    "command_id": command_id,
                    "language": language,
                    "payload_type": payload_type,
                    "region": region,
                    "voice_settings": voice_settings,
                },
                action_speak_params.ActionSpeakParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSpeakResponse,
        )

    async def stop(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionStopResponse:
        """
        Stop audio being played to all or some participants on a conference call.

        Args:
          call_control_ids: List of call control ids identifying participants the audio file should stop be
              played to. If not given, the audio will be stoped to the entire conference.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/stop",
            body=await async_maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_stop_params.ActionStopParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionStopResponse,
        )

    async def unhold(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str],
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnholdResponse:
        """
        Unhold a list of participants in a conference call

        Args:
          call_control_ids: List of unique identifiers and tokens for controlling the call. Enter each call
              control ID to be unheld.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/unhold",
            body=await async_maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_unhold_params.ActionUnholdParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnholdResponse,
        )

    async def unmute(
        self,
        id: str,
        *,
        call_control_ids: SequenceNotStr[str] | Omit = omit,
        region: Literal["Australia", "Europe", "Middle East", "US"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnmuteResponse:
        """
        Unmute a list of participants in a conference call

        Args:
          call_control_ids: List of unique identifiers and tokens for controlling the call. Enter each call
              control ID to be unmuted. When empty all participants will be unmuted.

          region: Region where the conference data is located. Defaults to the region defined in
              user's data locality settings (Europe or US).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/conferences/{id}/actions/unmute",
            body=await async_maybe_transform(
                {
                    "call_control_ids": call_control_ids,
                    "region": region,
                },
                action_unmute_params.ActionUnmuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnmuteResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.update = to_raw_response_wrapper(
            actions.update,
        )
        self.end_conference = to_raw_response_wrapper(
            actions.end_conference,
        )
        self.gather_dtmf_audio = to_raw_response_wrapper(
            actions.gather_dtmf_audio,
        )
        self.hold = to_raw_response_wrapper(
            actions.hold,
        )
        self.join = to_raw_response_wrapper(
            actions.join,
        )
        self.leave = to_raw_response_wrapper(
            actions.leave,
        )
        self.mute = to_raw_response_wrapper(
            actions.mute,
        )
        self.play = to_raw_response_wrapper(
            actions.play,
        )
        self.record_pause = to_raw_response_wrapper(
            actions.record_pause,
        )
        self.record_resume = to_raw_response_wrapper(
            actions.record_resume,
        )
        self.record_start = to_raw_response_wrapper(
            actions.record_start,
        )
        self.record_stop = to_raw_response_wrapper(
            actions.record_stop,
        )
        self.send_dtmf = to_raw_response_wrapper(
            actions.send_dtmf,
        )
        self.speak = to_raw_response_wrapper(
            actions.speak,
        )
        self.stop = to_raw_response_wrapper(
            actions.stop,
        )
        self.unhold = to_raw_response_wrapper(
            actions.unhold,
        )
        self.unmute = to_raw_response_wrapper(
            actions.unmute,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.update = async_to_raw_response_wrapper(
            actions.update,
        )
        self.end_conference = async_to_raw_response_wrapper(
            actions.end_conference,
        )
        self.gather_dtmf_audio = async_to_raw_response_wrapper(
            actions.gather_dtmf_audio,
        )
        self.hold = async_to_raw_response_wrapper(
            actions.hold,
        )
        self.join = async_to_raw_response_wrapper(
            actions.join,
        )
        self.leave = async_to_raw_response_wrapper(
            actions.leave,
        )
        self.mute = async_to_raw_response_wrapper(
            actions.mute,
        )
        self.play = async_to_raw_response_wrapper(
            actions.play,
        )
        self.record_pause = async_to_raw_response_wrapper(
            actions.record_pause,
        )
        self.record_resume = async_to_raw_response_wrapper(
            actions.record_resume,
        )
        self.record_start = async_to_raw_response_wrapper(
            actions.record_start,
        )
        self.record_stop = async_to_raw_response_wrapper(
            actions.record_stop,
        )
        self.send_dtmf = async_to_raw_response_wrapper(
            actions.send_dtmf,
        )
        self.speak = async_to_raw_response_wrapper(
            actions.speak,
        )
        self.stop = async_to_raw_response_wrapper(
            actions.stop,
        )
        self.unhold = async_to_raw_response_wrapper(
            actions.unhold,
        )
        self.unmute = async_to_raw_response_wrapper(
            actions.unmute,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.update = to_streamed_response_wrapper(
            actions.update,
        )
        self.end_conference = to_streamed_response_wrapper(
            actions.end_conference,
        )
        self.gather_dtmf_audio = to_streamed_response_wrapper(
            actions.gather_dtmf_audio,
        )
        self.hold = to_streamed_response_wrapper(
            actions.hold,
        )
        self.join = to_streamed_response_wrapper(
            actions.join,
        )
        self.leave = to_streamed_response_wrapper(
            actions.leave,
        )
        self.mute = to_streamed_response_wrapper(
            actions.mute,
        )
        self.play = to_streamed_response_wrapper(
            actions.play,
        )
        self.record_pause = to_streamed_response_wrapper(
            actions.record_pause,
        )
        self.record_resume = to_streamed_response_wrapper(
            actions.record_resume,
        )
        self.record_start = to_streamed_response_wrapper(
            actions.record_start,
        )
        self.record_stop = to_streamed_response_wrapper(
            actions.record_stop,
        )
        self.send_dtmf = to_streamed_response_wrapper(
            actions.send_dtmf,
        )
        self.speak = to_streamed_response_wrapper(
            actions.speak,
        )
        self.stop = to_streamed_response_wrapper(
            actions.stop,
        )
        self.unhold = to_streamed_response_wrapper(
            actions.unhold,
        )
        self.unmute = to_streamed_response_wrapper(
            actions.unmute,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.update = async_to_streamed_response_wrapper(
            actions.update,
        )
        self.end_conference = async_to_streamed_response_wrapper(
            actions.end_conference,
        )
        self.gather_dtmf_audio = async_to_streamed_response_wrapper(
            actions.gather_dtmf_audio,
        )
        self.hold = async_to_streamed_response_wrapper(
            actions.hold,
        )
        self.join = async_to_streamed_response_wrapper(
            actions.join,
        )
        self.leave = async_to_streamed_response_wrapper(
            actions.leave,
        )
        self.mute = async_to_streamed_response_wrapper(
            actions.mute,
        )
        self.play = async_to_streamed_response_wrapper(
            actions.play,
        )
        self.record_pause = async_to_streamed_response_wrapper(
            actions.record_pause,
        )
        self.record_resume = async_to_streamed_response_wrapper(
            actions.record_resume,
        )
        self.record_start = async_to_streamed_response_wrapper(
            actions.record_start,
        )
        self.record_stop = async_to_streamed_response_wrapper(
            actions.record_stop,
        )
        self.send_dtmf = async_to_streamed_response_wrapper(
            actions.send_dtmf,
        )
        self.speak = async_to_streamed_response_wrapper(
            actions.speak,
        )
        self.stop = async_to_streamed_response_wrapper(
            actions.stop,
        )
        self.unhold = async_to_streamed_response_wrapper(
            actions.unhold,
        )
        self.unmute = async_to_streamed_response_wrapper(
            actions.unmute,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.feature import Feature
from .shared.cost_information import CostInformation
from .shared.region_information import RegionInformation
from .shared.available_phone_numbers_metadata import AvailablePhoneNumbersMetadata

__all__ = ["AvailablePhoneNumberListResponse", "Data"]


class Data(BaseModel):
    best_effort: Optional[bool] = None
    """
    Specifies whether the phone number is an exact match based on the search
    criteria or not.
    """

    cost_information: Optional[CostInformation] = None

    features: Optional[List[Feature]] = None

    phone_number: Optional[str] = None

    quickship: Optional[bool] = None
    """
    Specifies whether the phone number can receive calls immediately after purchase
    or not.
    """

    record_type: Optional[Literal["available_phone_number"]] = None

    region_information: Optional[List[RegionInformation]] = None

    reservable: Optional[bool] = None
    """Specifies whether the phone number can be reserved before purchase or not."""

    vanity_format: Optional[str] = None


class AvailablePhoneNumberListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[AvailablePhoneNumbersMetadata] = None

    metadata: Optional[AvailablePhoneNumbersMetadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel
from .country_coverage import CountryCoverage

__all__ = ["CountryCoverageRetrieveResponse"]


class CountryCoverageRetrieveResponse(BaseModel):
    data: Optional[Dict[str, CountryCoverage]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PortingOrderEndUserLocationParam"]


class PortingOrderEndUserLocationParam(TypedDict, total=False):
    administrative_area: Optional[str]
    """State, province, or similar of billing address"""

    country_code: Optional[str]
    """ISO3166-1 alpha-2 country code of billing address"""

    extended_address: Optional[str]
    """Second line of billing address"""

    locality: Optional[str]
    """City or municipality of billing address"""

    postal_code: Optional[str]
    """Postal Code of billing address"""

    street_address: Optional[str]
    """First line of billing address"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .requirement_group import RequirementGroup

__all__ = ["RequirementGroupListResponse"]

RequirementGroupListResponse: TypeAlias = List[RequirementGroup]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionGatherParams"]


class ActionGatherParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    gather_id: str
    """An id that will be sent back in the corresponding `call.gather.ended` webhook.

    Will be randomly generated if not specified.
    """

    initial_timeout_millis: int
    """The number of milliseconds to wait for the first DTMF."""

    inter_digit_timeout_millis: int
    """The number of milliseconds to wait for input between digits."""

    maximum_digits: int
    """The maximum number of digits to fetch.

    This parameter has a maximum value of 128.
    """

    minimum_digits: int
    """The minimum number of digits to fetch. This parameter has a minimum value of 1."""

    terminating_digit: str
    """
    The digit used to terminate input if fewer than `maximum_digits` digits have
    been gathered.
    """

    timeout_millis: int
    """The number of milliseconds to wait to complete the request."""

    valid_digits: str
    """A list of all digits accepted as valid."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .phone_number_blocks import (
    PhoneNumberBlocksResource,
    AsyncPhoneNumberBlocksResource,
    PhoneNumberBlocksResourceWithRawResponse,
    AsyncPhoneNumberBlocksResourceWithRawResponse,
    PhoneNumberBlocksResourceWithStreamingResponse,
    AsyncPhoneNumberBlocksResourceWithStreamingResponse,
)

__all__ = [
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "PhoneNumberBlocksResource",
    "AsyncPhoneNumberBlocksResource",
    "PhoneNumberBlocksResourceWithRawResponse",
    "AsyncPhoneNumberBlocksResourceWithRawResponse",
    "PhoneNumberBlocksResourceWithStreamingResponse",
    "AsyncPhoneNumberBlocksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_participant_joined import ConferenceParticipantJoined

__all__ = ["ConferenceParticipantJoinedWebhookEvent"]


class ConferenceParticipantJoinedWebhookEvent(BaseModel):
    data: Optional[ConferenceParticipantJoined] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import json
import logging
from types import TracebackType
from typing import TYPE_CHECKING, Any, Dict, Iterator, cast
from typing_extensions import Literal, AsyncIterator

import httpx
from pydantic import BaseModel

from ..types import text_to_speech_generate_params, text_to_speech_list_voices_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._models import construct_type_unchecked
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._exceptions import TelnyxError
from .._base_client import _merge_mappings, make_request_options
from ..types.stream_client_event import StreamClientEvent
from ..types.stream_server_event import StreamServerEvent
from ..types.stream_client_event_param import StreamClientEventParam
from ..types.websocket_connection_options import WebsocketConnectionOptions
from ..types.text_to_speech_generate_response import TextToSpeechGenerateResponse
from ..types.text_to_speech_list_voices_response import TextToSpeechListVoicesResponse

if TYPE_CHECKING:
    from websockets.sync.client import ClientConnection as WebsocketConnection
    from websockets.asyncio.client import ClientConnection as AsyncWebsocketConnection

    from .._client import Telnyx, AsyncTelnyx

__all__ = ["TextToSpeechResource", "AsyncTextToSpeechResource"]

log: logging.Logger = logging.getLogger(__name__)


class TextToSpeechResource(SyncAPIResource):
    """Text to speech streaming command operations"""

    @cached_property
    def with_raw_response(self) -> TextToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TextToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TextToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TextToSpeechResourceWithStreamingResponse(self)

    def generate(
        self,
        *,
        aws: text_to_speech_generate_params.Aws | Omit = omit,
        azure: text_to_speech_generate_params.Azure | Omit = omit,
        disable_cache: bool | Omit = omit,
        elevenlabs: text_to_speech_generate_params.Elevenlabs | Omit = omit,
        inworld: Dict[str, object] | Omit = omit,
        language: str | Omit = omit,
        minimax: text_to_speech_generate_params.Minimax | Omit = omit,
        output_type: Literal["binary_output", "base64_output"] | Omit = omit,
        provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
        | Omit = omit,
        resemble: text_to_speech_generate_params.Resemble | Omit = omit,
        rime: text_to_speech_generate_params.Rime | Omit = omit,
        telnyx: text_to_speech_generate_params.Telnyx | Omit = omit,
        text: str | Omit = omit,
        text_type: Literal["text", "ssml"] | Omit = omit,
        voice: str | Omit = omit,
        voice_settings: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TextToSpeechGenerateResponse:
        """Generate synthesized speech audio from text input.

        Returns audio in the
        requested format (binary audio stream, base64-encoded JSON, or an audio URL for
        later retrieval).

        Authentication is provided via the standard `Authorization: Bearer <API_KEY>`
        header.

        The `voice` parameter provides a convenient shorthand to specify provider,
        model, and voice in a single string (e.g. `telnyx.NaturalHD.Alloy`).
        Alternatively, specify `provider` explicitly along with provider-specific
        parameters.

        Supported providers: `aws`, `telnyx`, `azure`, `elevenlabs`, `minimax`, `rime`,
        `resemble`, `inworld`.

        Args:
          aws: AWS Polly provider-specific parameters.

          azure: Azure Cognitive Services provider-specific parameters.

          disable_cache: When `true`, bypass the audio cache and generate fresh audio.

          elevenlabs: ElevenLabs provider-specific parameters.

          inworld: Inworld provider-specific parameters.

          language: Language code (e.g. `en-US`). Usage varies by provider.

          minimax: Minimax provider-specific parameters.

          output_type: Determines the response format. `binary_output` returns raw audio bytes,
              `base64_output` returns base64-encoded audio in JSON.

          provider: TTS provider. Required unless `voice` is provided.

          resemble: Resemble AI provider-specific parameters.

          rime: Rime provider-specific parameters.

          telnyx: Telnyx provider-specific parameters.

          text: The text to convert to speech.

          text_type: Text type. Use `ssml` for SSML-formatted input (supported by AWS and Azure).

          voice: Voice identifier in the format `provider.model_id.voice_id` or
              `provider.voice_id`. Examples: `telnyx.NaturalHD.Alloy`,
              `azure.en-US-AvaMultilingualNeural`, `aws.Polly.Generative.Lucia`. When
              provided, `provider`, `model_id`, and `voice_id` are extracted automatically and
              take precedence over individual parameters.

          voice_settings: Provider-specific voice settings. Contents vary by provider — see
              provider-specific parameter objects below.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/text-to-speech/speech",
            body=maybe_transform(
                {
                    "aws": aws,
                    "azure": azure,
                    "disable_cache": disable_cache,
                    "elevenlabs": elevenlabs,
                    "inworld": inworld,
                    "language": language,
                    "minimax": minimax,
                    "output_type": output_type,
                    "provider": provider,
                    "resemble": resemble,
                    "rime": rime,
                    "telnyx": telnyx,
                    "text": text,
                    "text_type": text_type,
                    "voice": voice,
                    "voice_settings": voice_settings,
                },
                text_to_speech_generate_params.TextToSpeechGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TextToSpeechGenerateResponse,
        )

    def list_voices(
        self,
        *,
        api_key: str | Omit = omit,
        provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TextToSpeechListVoicesResponse:
        """Retrieve a list of available voices from one or all TTS providers.

        When
        `provider` is specified, returns voices for that provider only. Otherwise,
        returns voices from all providers.

        Some providers (ElevenLabs, Resemble) require an API key to list voices.

        Args:
          api_key: API key for providers that require one to list voices (e.g. ElevenLabs).

          provider: Filter voices by provider. If omitted, voices from all providers are returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/text-to-speech/voices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "api_key": api_key,
                        "provider": provider,
                    },
                    text_to_speech_list_voices_params.TextToSpeechListVoicesParams,
                ),
            ),
            cast_to=TextToSpeechListVoicesResponse,
        )

    def stream(
        self,
        extra_query: Query = {},
        extra_headers: Headers = {},
        websocket_connection_options: WebsocketConnectionOptions = {},
    ) -> TextToSpeechResourceConnectionManager:
        return TextToSpeechResourceConnectionManager(
            client=self._client,
            extra_query=extra_query,
            extra_headers=extra_headers,
            websocket_connection_options=websocket_connection_options,
        )


class AsyncTextToSpeechResource(AsyncAPIResource):
    """Text to speech streaming command operations"""

    @cached_property
    def with_raw_response(self) -> AsyncTextToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTextToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTextToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTextToSpeechResourceWithStreamingResponse(self)

    async def generate(
        self,
        *,
        aws: text_to_speech_generate_params.Aws | Omit = omit,
        azure: text_to_speech_generate_params.Azure | Omit = omit,
        disable_cache: bool | Omit = omit,
        elevenlabs: text_to_speech_generate_params.Elevenlabs | Omit = omit,
        inworld: Dict[str, object] | Omit = omit,
        language: str | Omit = omit,
        minimax: text_to_speech_generate_params.Minimax | Omit = omit,
        output_type: Literal["binary_output", "base64_output"] | Omit = omit,
        provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
        | Omit = omit,
        resemble: text_to_speech_generate_params.Resemble | Omit = omit,
        rime: text_to_speech_generate_params.Rime | Omit = omit,
        telnyx: text_to_speech_generate_params.Telnyx | Omit = omit,
        text: str | Omit = omit,
        text_type: Literal["text", "ssml"] | Omit = omit,
        voice: str | Omit = omit,
        voice_settings: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TextToSpeechGenerateResponse:
        """Generate synthesized speech audio from text input.

        Returns audio in the
        requested format (binary audio stream, base64-encoded JSON, or an audio URL for
        later retrieval).

        Authentication is provided via the standard `Authorization: Bearer <API_KEY>`
        header.

        The `voice` parameter provides a convenient shorthand to specify provider,
        model, and voice in a single string (e.g. `telnyx.NaturalHD.Alloy`).
        Alternatively, specify `provider` explicitly along with provider-specific
        parameters.

        Supported providers: `aws`, `telnyx`, `azure`, `elevenlabs`, `minimax`, `rime`,
        `resemble`, `inworld`.

        Args:
          aws: AWS Polly provider-specific parameters.

          azure: Azure Cognitive Services provider-specific parameters.

          disable_cache: When `true`, bypass the audio cache and generate fresh audio.

          elevenlabs: ElevenLabs provider-specific parameters.

          inworld: Inworld provider-specific parameters.

          language: Language code (e.g. `en-US`). Usage varies by provider.

          minimax: Minimax provider-specific parameters.

          output_type: Determines the response format. `binary_output` returns raw audio bytes,
              `base64_output` returns base64-encoded audio in JSON.

          provider: TTS provider. Required unless `voice` is provided.

          resemble: Resemble AI provider-specific parameters.

          rime: Rime provider-specific parameters.

          telnyx: Telnyx provider-specific parameters.

          text: The text to convert to speech.

          text_type: Text type. Use `ssml` for SSML-formatted input (supported by AWS and Azure).

          voice: Voice identifier in the format `provider.model_id.voice_id` or
              `provider.voice_id`. Examples: `telnyx.NaturalHD.Alloy`,
              `azure.en-US-AvaMultilingualNeural`, `aws.Polly.Generative.Lucia`. When
              provided, `provider`, `model_id`, and `voice_id` are extracted automatically and
              take precedence over individual parameters.

          voice_settings: Provider-specific voice settings. Contents vary by provider — see
              provider-specific parameter objects below.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/text-to-speech/speech",
            body=await async_maybe_transform(
                {
                    "aws": aws,
                    "azure": azure,
                    "disable_cache": disable_cache,
                    "elevenlabs": elevenlabs,
                    "inworld": inworld,
                    "language": language,
                    "minimax": minimax,
                    "output_type": output_type,
                    "provider": provider,
                    "resemble": resemble,
                    "rime": rime,
                    "telnyx": telnyx,
                    "text": text,
                    "text_type": text_type,
                    "voice": voice,
                    "voice_settings": voice_settings,
                },
                text_to_speech_generate_params.TextToSpeechGenerateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TextToSpeechGenerateResponse,
        )

    async def list_voices(
        self,
        *,
        api_key: str | Omit = omit,
        provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TextToSpeechListVoicesResponse:
        """Retrieve a list of available voices from one or all TTS providers.

        When
        `provider` is specified, returns voices for that provider only. Otherwise,
        returns voices from all providers.

        Some providers (ElevenLabs, Resemble) require an API key to list voices.

        Args:
          api_key: API key for providers that require one to list voices (e.g. ElevenLabs).

          provider: Filter voices by provider. If omitted, voices from all providers are returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/text-to-speech/voices",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "api_key": api_key,
                        "provider": provider,
                    },
                    text_to_speech_list_voices_params.TextToSpeechListVoicesParams,
                ),
            ),
            cast_to=TextToSpeechListVoicesResponse,
        )

    def stream(
        self,
        extra_query: Query = {},
        extra_headers: Headers = {},
        websocket_connection_options: WebsocketConnectionOptions = {},
    ) -> AsyncTextToSpeechResourceConnectionManager:
        return AsyncTextToSpeechResourceConnectionManager(
            client=self._client,
            extra_query=extra_query,
            extra_headers=extra_headers,
            websocket_connection_options=websocket_connection_options,
        )


class TextToSpeechResourceWithRawResponse:
    def __init__(self, text_to_speech: TextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.generate = to_raw_response_wrapper(
            text_to_speech.generate,
        )
        self.list_voices = to_raw_response_wrapper(
            text_to_speech.list_voices,
        )


class AsyncTextToSpeechResourceWithRawResponse:
    def __init__(self, text_to_speech: AsyncTextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.generate = async_to_raw_response_wrapper(
            text_to_speech.generate,
        )
        self.list_voices = async_to_raw_response_wrapper(
            text_to_speech.list_voices,
        )


class TextToSpeechResourceWithStreamingResponse:
    def __init__(self, text_to_speech: TextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.generate = to_streamed_response_wrapper(
            text_to_speech.generate,
        )
        self.list_voices = to_streamed_response_wrapper(
            text_to_speech.list_voices,
        )


class AsyncTextToSpeechResourceWithStreamingResponse:
    def __init__(self, text_to_speech: AsyncTextToSpeechResource) -> None:
        self._text_to_speech = text_to_speech

        self.generate = async_to_streamed_response_wrapper(
            text_to_speech.generate,
        )
        self.list_voices = async_to_streamed_response_wrapper(
            text_to_speech.list_voices,
        )


class AsyncTextToSpeechResourceConnection:
    """Represents a live WebSocket connection to the TextToSpeech API"""

    _connection: AsyncWebsocketConnection

    def __init__(self, connection: AsyncWebsocketConnection) -> None:
        self._connection = connection

    async def __aiter__(self) -> AsyncIterator[StreamServerEvent]:
        """
        An infinite-iterator that will continue to yield events until
        the connection is closed.
        """
        from websockets.exceptions import ConnectionClosedOK

        try:
            while True:
                yield await self.recv()
        except ConnectionClosedOK:
            return

    async def recv(self) -> StreamServerEvent:
        """
        Receive the next message from the connection and parses it into a `StreamServerEvent` object.

        Canceling this method is safe. There's no risk of losing data.
        """
        return self.parse_event(await self.recv_bytes())

    async def recv_bytes(self) -> bytes:
        """Receive the next message from the connection as raw bytes.

        Canceling this method is safe. There's no risk of losing data.

        If you want to parse the message into a `StreamServerEvent` object like `.recv()` does,
        then you can call `.parse_event(data)`.
        """
        message = await self._connection.recv(decode=False)
        log.debug(f"Received websocket message: %s", message)
        return message

    async def send(self, event: StreamClientEvent | StreamClientEventParam) -> None:
        data = (
            event.to_json(use_api_names=True, exclude_defaults=True, exclude_unset=True)
            if isinstance(event, BaseModel)
            else json.dumps(await async_maybe_transform(event, StreamClientEventParam))
        )
        await self._connection.send(data)

    async def close(self, *, code: int = 1000, reason: str = "") -> None:
        await self._connection.close(code=code, reason=reason)

    def parse_event(self, data: str | bytes) -> StreamServerEvent:
        """
        Converts a raw `str` or `bytes` message into a `StreamServerEvent` object.

        This is helpful if you're using `.recv_bytes()`.
        """
        return cast(
            StreamServerEvent, construct_type_unchecked(value=json.loads(data), type_=cast(Any, StreamServerEvent))
        )


class AsyncTextToSpeechResourceConnectionManager:
    """
    Context manager over a `AsyncTextToSpeechResourceConnection` that is returned by `text_to_speech.stream()`

    This context manager ensures that the connection will be closed when it exits.

    ---

    Note that if your application doesn't work well with the context manager approach then you
    can call the `.enter()` method directly to initiate a connection.

    **Warning**: You must remember to close the connection with `.close()`.

    ```py
    connection = await client.text_to_speech.stream(...).enter()
    # ...
    await connection.close()
    ```
    """

    def __init__(
        self,
        *,
        client: AsyncTelnyx,
        extra_query: Query,
        extra_headers: Headers,
        websocket_connection_options: WebsocketConnectionOptions,
    ) -> None:
        self.__client = client
        self.__connection: AsyncTextToSpeechResourceConnection | None = None
        self.__extra_query = extra_query
        self.__extra_headers = extra_headers
        self.__websocket_connection_options = websocket_connection_options

    async def __aenter__(self) -> AsyncTextToSpeechResourceConnection:
        """
        👋 If your application doesn't work well with the context manager approach then you
        can call this method directly to initiate a connection.

        **Warning**: You must remember to close the connection with `.close()`.

        ```py
        connection = await client.text_to_speech.stream(...).enter()
        # ...
        await connection.close()
        ```
        """
        try:
            from websockets.asyncio.client import connect
        except ImportError as exc:
            raise TelnyxError("You need to install `telnyx[websockets]` to use this method") from exc

        url = self._prepare_url().copy_with(
            params={
                **self.__client.base_url.params,
                **self.__extra_query,
            },
        )
        log.debug("Connecting to %s", url)
        if self.__websocket_connection_options:
            log.debug("Connection options: %s", self.__websocket_connection_options)

        self.__connection = AsyncTextToSpeechResourceConnection(
            await connect(
                str(url),
                user_agent_header=self.__client.user_agent,
                additional_headers=_merge_mappings(
                    {
                        **self.__client.auth_headers,
                    },
                    self.__extra_headers,
                ),
                **self.__websocket_connection_options,
            )
        )

        return self.__connection

    enter = __aenter__

    def _prepare_url(self) -> httpx.URL:
        if self.__client.websocket_base_url is not None:
            base_url = httpx.URL(self.__client.websocket_base_url)
        else:
            base_url = self.__client._base_url.copy_with(scheme="wss")

        merge_raw_path = base_url.raw_path.rstrip(b"/") + b"/text-to-speech/speech"
        return base_url.copy_with(raw_path=merge_raw_path)

    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        if self.__connection is not None:
            await self.__connection.close()


class TextToSpeechResourceConnection:
    """Represents a live WebSocket connection to the TextToSpeech API"""

    _connection: WebsocketConnection

    def __init__(self, connection: WebsocketConnection) -> None:
        self._connection = connection

    def __iter__(self) -> Iterator[StreamServerEvent]:
        """
        An infinite-iterator that will continue to yield events until
        the connection is closed.
        """
        from websockets.exceptions import ConnectionClosedOK

        try:
            while True:
                yield self.recv()
        except ConnectionClosedOK:
            return

    def recv(self) -> StreamServerEvent:
        """
        Receive the next message from the connection and parses it into a `StreamServerEvent` object.

        Canceling this method is safe. There's no risk of losing data.
        """
        return self.parse_event(self.recv_bytes())

    def recv_bytes(self) -> bytes:
        """Receive the next message from the connection as raw bytes.

        Canceling this method is safe. There's no risk of losing data.

        If you want to parse the message into a `StreamServerEvent` object like `.recv()` does,
        then you can call `.parse_event(data)`.
        """
        message = self._connection.recv(decode=False)
        log.debug(f"Received websocket message: %s", message)
        return message

    def send(self, event: StreamClientEvent | StreamClientEventParam) -> None:
        data = (
            event.to_json(use_api_names=True, exclude_defaults=True, exclude_unset=True)
            if isinstance(event, BaseModel)
            else json.dumps(maybe_transform(event, StreamClientEventParam))
        )
        self._connection.send(data)

    def close(self, *, code: int = 1000, reason: str = "") -> None:
        self._connection.close(code=code, reason=reason)

    def parse_event(self, data: str | bytes) -> StreamServerEvent:
        """
        Converts a raw `str` or `bytes` message into a `StreamServerEvent` object.

        This is helpful if you're using `.recv_bytes()`.
        """
        return cast(
            StreamServerEvent, construct_type_unchecked(value=json.loads(data), type_=cast(Any, StreamServerEvent))
        )


class TextToSpeechResourceConnectionManager:
    """
    Context manager over a `TextToSpeechResourceConnection` that is returned by `text_to_speech.stream()`

    This context manager ensures that the connection will be closed when it exits.

    ---

    Note that if your application doesn't work well with the context manager approach then you
    can call the `.enter()` method directly to initiate a connection.

    **Warning**: You must remember to close the connection with `.close()`.

    ```py
    connection = client.text_to_speech.stream(...).enter()
    # ...
    connection.close()
    ```
    """

    def __init__(
        self,
        *,
        client: Telnyx,
        extra_query: Query,
        extra_headers: Headers,
        websocket_connection_options: WebsocketConnectionOptions,
    ) -> None:
        self.__client = client
        self.__connection: TextToSpeechResourceConnection | None = None
        self.__extra_query = extra_query
        self.__extra_headers = extra_headers
        self.__websocket_connection_options = websocket_connection_options

    def __enter__(self) -> TextToSpeechResourceConnection:
        """
        👋 If your application doesn't work well with the context manager approach then you
        can call this method directly to initiate a connection.

        **Warning**: You must remember to close the connection with `.close()`.

        ```py
        connection = client.text_to_speech.stream(...).enter()
        # ...
        connection.close()
        ```
        """
        try:
            from websockets.sync.client import connect
        except ImportError as exc:
            raise TelnyxError("You need to install `telnyx[websockets]` to use this method") from exc

        url = self._prepare_url().copy_with(
            params={
                **self.__client.base_url.params,
                **self.__extra_query,
            },
        )
        log.debug("Connecting to %s", url)
        if self.__websocket_connection_options:
            log.debug("Connection options: %s", self.__websocket_connection_options)

        self.__connection = TextToSpeechResourceConnection(
            connect(
                str(url),
                user_agent_header=self.__client.user_agent,
                additional_headers=_merge_mappings(
                    {
                        **self.__client.auth_headers,
                    },
                    self.__extra_headers,
                ),
                **self.__websocket_connection_options,
            )
        )

        return self.__connection

    enter = __enter__

    def _prepare_url(self) -> httpx.URL:
        if self.__client.websocket_base_url is not None:
            base_url = httpx.URL(self.__client.websocket_base_url)
        else:
            base_url = self.__client._base_url.copy_with(scheme="wss")

        merge_raw_path = base_url.raw_path.rstrip(b"/") + b"/text-to-speech/speech"
        return base_url.copy_with(raw_path=merge_raw_path)

    def __exit__(
        self, exc_type: type[BaseException] | None, exc: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        if self.__connection is not None:
            self.__connection.close()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.types.storage.buckets import (
    UsageGetAPIUsageResponse,
    UsageGetBucketUsageResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsage:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_api_usage(self, client: Telnyx) -> None:
        usage = client.storage.buckets.usage.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )
        assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_api_usage(self, client: Telnyx) -> None:
        response = client.storage.buckets.usage.with_raw_response.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_api_usage(self, client: Telnyx) -> None:
        with client.storage.buckets.usage.with_streaming_response.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_api_usage(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            client.storage.buckets.usage.with_raw_response.get_api_usage(
                bucket_name="",
                filter={
                    "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_bucket_usage(self, client: Telnyx) -> None:
        usage = client.storage.buckets.usage.get_bucket_usage(
            "",
        )
        assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_bucket_usage(self, client: Telnyx) -> None:
        response = client.storage.buckets.usage.with_raw_response.get_bucket_usage(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_bucket_usage(self, client: Telnyx) -> None:
        with client.storage.buckets.usage.with_streaming_response.get_bucket_usage(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_bucket_usage(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            client.storage.buckets.usage.with_raw_response.get_bucket_usage(
                "",
            )


class TestAsyncUsage:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_api_usage(self, async_client: AsyncTelnyx) -> None:
        usage = await async_client.storage.buckets.usage.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )
        assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_api_usage(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.storage.buckets.usage.with_raw_response.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_api_usage(self, async_client: AsyncTelnyx) -> None:
        async with async_client.storage.buckets.usage.with_streaming_response.get_api_usage(
            bucket_name="",
            filter={
                "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageGetAPIUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_api_usage(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            await async_client.storage.buckets.usage.with_raw_response.get_api_usage(
                bucket_name="",
                filter={
                    "end_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "start_time": parse_datetime("2019-12-27T18:11:19.117Z"),
                },
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_bucket_usage(self, async_client: AsyncTelnyx) -> None:
        usage = await async_client.storage.buckets.usage.get_bucket_usage(
            "",
        )
        assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_bucket_usage(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.storage.buckets.usage.with_raw_response.get_bucket_usage(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = await response.parse()
        assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_bucket_usage(self, async_client: AsyncTelnyx) -> None:
        async with async_client.storage.buckets.usage.with_streaming_response.get_bucket_usage(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageGetBucketUsageResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_bucket_usage(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            await async_client.storage.buckets.usage.with_raw_response.get_bucket_usage(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["BrandGetSMSOtpByReferenceParams"]


class BrandGetSMSOtpByReferenceParams(TypedDict, total=False):
    brand_id: Annotated[str, PropertyInfo(alias="brandId")]
    """Filter by Brand ID for easier lookup in portal applications"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .faxes import (
    FaxesResource,
    AsyncFaxesResource,
    FaxesResourceWithRawResponse,
    AsyncFaxesResourceWithRawResponse,
    FaxesResourceWithStreamingResponse,
    AsyncFaxesResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "FaxesResource",
    "AsyncFaxesResource",
    "FaxesResourceWithRawResponse",
    "AsyncFaxesResourceWithRawResponse",
    "FaxesResourceWithStreamingResponse",
    "AsyncFaxesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel
from .stt_detail_report_response import SttDetailReportResponse

__all__ = ["SpeechToTextDeleteResponse"]


class SpeechToTextDeleteResponse(BaseModel):
    data: Optional[SttDetailReportResponse] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.addresses import action_validate_params, action_accept_suggestions_params
from ...types.addresses.action_validate_response import ActionValidateResponse
from ...types.addresses.action_accept_suggestions_response import ActionAcceptSuggestionsResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    """Operations to work with Address records.

    Address records are emergency-validated addresses meant to be associated with phone numbers. They are validated for emergency usage purposes at creation time, although you may validate them separately with a custom workflow using the ValidateAddress operation separately. Address records are not usable for physical orders, such as for Telnyx SIM cards, please use UserAddress for that. It is not possible to entirely skip emergency service validation for Address records; if an emergency provider for a phone number rejects the address then it cannot be used on a phone number. To prevent records from getting out of sync, Address records are immutable and cannot be altered once created. If you realize you need to alter an address, a new record must be created with the differing address.
    """

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def accept_suggestions(
        self,
        address_uuid: str,
        *,
        id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionAcceptSuggestionsResponse:
        """
        Accepts this address suggestion as a new emergency address for Operator Connect
        and finishes the uploads of the numbers associated with it to Microsoft.

        Args:
          id: The ID of the address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_uuid:
            raise ValueError(f"Expected a non-empty value for `address_uuid` but received {address_uuid!r}")
        return self._post(
            f"/addresses/{address_uuid}/actions/accept_suggestions",
            body=maybe_transform({"id": id}, action_accept_suggestions_params.ActionAcceptSuggestionsParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionAcceptSuggestionsResponse,
        )

    def validate(
        self,
        *,
        country_code: str,
        postal_code: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        extended_address: str | Omit = omit,
        locality: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionValidateResponse:
        """
        Validates an address for emergency services.

        Args:
          country_code: The two-character (ISO 3166-1 alpha-2) country code of the address.

          postal_code: The postal code of the address.

          street_address: The primary street address information about the address.

          administrative_area: The locality of the address. For US addresses, this corresponds to the state of
              the address.

          extended_address: Additional street address information about the address such as, but not limited
              to, unit number or apartment number.

          locality: The locality of the address. For US addresses, this corresponds to the city of
              the address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/addresses/actions/validate",
            body=maybe_transform(
                {
                    "country_code": country_code,
                    "postal_code": postal_code,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "extended_address": extended_address,
                    "locality": locality,
                },
                action_validate_params.ActionValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionValidateResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    """Operations to work with Address records.

    Address records are emergency-validated addresses meant to be associated with phone numbers. They are validated for emergency usage purposes at creation time, although you may validate them separately with a custom workflow using the ValidateAddress operation separately. Address records are not usable for physical orders, such as for Telnyx SIM cards, please use UserAddress for that. It is not possible to entirely skip emergency service validation for Address records; if an emergency provider for a phone number rejects the address then it cannot be used on a phone number. To prevent records from getting out of sync, Address records are immutable and cannot be altered once created. If you realize you need to alter an address, a new record must be created with the differing address.
    """

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def accept_suggestions(
        self,
        address_uuid: str,
        *,
        id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionAcceptSuggestionsResponse:
        """
        Accepts this address suggestion as a new emergency address for Operator Connect
        and finishes the uploads of the numbers associated with it to Microsoft.

        Args:
          id: The ID of the address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not address_uuid:
            raise ValueError(f"Expected a non-empty value for `address_uuid` but received {address_uuid!r}")
        return await self._post(
            f"/addresses/{address_uuid}/actions/accept_suggestions",
            body=await async_maybe_transform(
                {"id": id}, action_accept_suggestions_params.ActionAcceptSuggestionsParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionAcceptSuggestionsResponse,
        )

    async def validate(
        self,
        *,
        country_code: str,
        postal_code: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        extended_address: str | Omit = omit,
        locality: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionValidateResponse:
        """
        Validates an address for emergency services.

        Args:
          country_code: The two-character (ISO 3166-1 alpha-2) country code of the address.

          postal_code: The postal code of the address.

          street_address: The primary street address information about the address.

          administrative_area: The locality of the address. For US addresses, this corresponds to the state of
              the address.

          extended_address: Additional street address information about the address such as, but not limited
              to, unit number or apartment number.

          locality: The locality of the address. For US addresses, this corresponds to the city of
              the address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/addresses/actions/validate",
            body=await async_maybe_transform(
                {
                    "country_code": country_code,
                    "postal_code": postal_code,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "extended_address": extended_address,
                    "locality": locality,
                },
                action_validate_params.ActionValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionValidateResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.accept_suggestions = to_raw_response_wrapper(
            actions.accept_suggestions,
        )
        self.validate = to_raw_response_wrapper(
            actions.validate,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.accept_suggestions = async_to_raw_response_wrapper(
            actions.accept_suggestions,
        )
        self.validate = async_to_raw_response_wrapper(
            actions.validate,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.accept_suggestions = to_streamed_response_wrapper(
            actions.accept_suggestions,
        )
        self.validate = to_streamed_response_wrapper(
            actions.validate,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.accept_suggestions = async_to_streamed_response_wrapper(
            actions.accept_suggestions,
        )
        self.validate = async_to_streamed_response_wrapper(
            actions.validate,
        )
